//**********************************************************//
//*  Calculate Diode Potential from Doping Profile         *//
//*  Also calculate induced signal (broken code...)        *//
//*                                    (c) PeFi 2008-2015  *//
//**********************************************************//
// Possible Improvements:                                   //
// - include real ionization probability                    //
// - plot gain vs. bias voltage                             //
// - improve statistics by drifting several charges         //
//**********************************************************//

//**********************************************************//
// This code has been developed for Borland C++ Builder     //
// with a graphical User Interface & graphical output       //
// -> Ignore all related commands.                          //
// Main code is in function Iterate()                       //
//**********************************************************//

#include <vcl>
#include <list>
#include <algorithm>
#include <math>
#pragma hdrstop

#include "MainUnit.h"
#pragma package(smart_init)
#pragma resource "*.dfm"

// Units:
// all dimensions are in um
// all times      are in ns
// all voltages   are in V
// all fields     are in V/um
// all velocities are in um/ns
// all currents   are in nA

// Types
typedef double Treal;                          // data type for all real numbers
typedef enum {ELECTRON, HOLE} TParticleType;   // for drift simulation
typedef struct {                               // Used in drift section
  TParticleType type;                          // ELECTRON or HOLE
  float         pos;                           // position
} TParticle;

// Physical Constants
const Treal C_q     = 1.60217646e-19;          // elementary charge [C]
const Treal C_k     = 1.3806503e-23;           // Boltzmann [J / K]
const Treal C_epsSi = 11.9;                    // dielectric constant of Si
const Treal C_eps0  = 8.8541878176e-18;        // in F/um
const Treal C_ni    = 1.01e-2;                 // in [1/um^3]

const Treal C_T     = 300.0;                   // room temperature [K]

// Derived constants to simplify code
const Treal C_Uth   = C_k * C_T / C_q;         // thermal voltage [V]
const Treal C_eps   = C_eps0 * C_epsSi;        // eps_si * eps_0
const Treal C_C     = 2 * C_q * C_ni / C_eps;  // see script eq. (10), used for speed

// Constants
const int   N       = 400;                     // discretization steps in potential calculations
const float tmax    = 10.0;                    // duration of drift simulation (ns)
const float dt      = 0.005;                   // time step in drift simulation (ns)

// Global Variables
TForm1 *Form1;                                 // the (graphics) form. Cbuilder specific!

Treal T;                                       // detector thickness
Treal V, Vp;                                   // external bias voltage, rescaled
Treal D  [N+1];                                // doping profile
Treal Phi[N+1];                                // potential
Treal E  [N+1];                                // field
Treal a;                                       // precalculated value

// Functions

inline Treal i2x(int i)                        // convert index in array to position
{
  return double(i) / double(N) * T;
}

inline int x2i(Treal x)                        // convert position to index in array
{
  if (x<=0)      return 0;
  else if (x>=T) return N;
  else           return x / T * (Treal) N;
}

float IonizationProbability(float E)           // ionization probability as function of field
{                                              // this is just a guess to get a
  return (E>8) ? 0.9 : 0;                      // reasonable result. Improve this!
//  return (E>3.8) ? 0.9 : 0;
}

float vdrift(TParticleType type, float Field)  // Drift velocity with approximative formula
{
  return (type==HOLE)                          // Field in V/um, v in um/ns
    ? - 83.4472 / (1 + 1.79881  / Field)
    :   107.052 / (1 + 0.698006 / Field);
}

void PlotPotential(TChart * chart)
{
  chart->Series[0]->Clear();
  chart->Title->Text->Clear();
  chart->Title->Text->Add("Electrostatic Potential [V]");;
  for (int i=0; i<=N; i++) {
    chart->Series[0]->AddXY(i2x(i),Phi[i]-Phi[0],"",clRed);
  }
}

void PlotField(TChart * chart)
{
  chart->Title->Text->Clear();
  chart->Title->Text->Add("Field [V/um]");;
  chart->Series[0]->Clear();
  for (int i=1; i<N; i++) {
    chart->Series[0]->AddXY(i2x(i),E[i],"",clGreen);
  }
}

void PlotCarrierDensity(TChart * chart)
{
  chart->Title->Text->Clear();
  chart->Title->Text->Add("|Carrier Density| [atoms/um3]");;
  chart->Series[0]->Clear();
  for (int i=1; i<N; i++) {    // use eq. (22) and (23)
    float val = C_ni * (exp(-Phi[i]/C_Uth-Vp) - exp(Phi[i]/C_Uth-Vp));
    chart->Series[0]->AddXY(i2x(i), fabs(val), "", clGreen);
  }
}


typedef enum {LEFT, RIGHT} TImplantSide;

void Diffuse (TImplantSide side, double dose, double depth)
{
  for (int i=0; i<=N; i++) {                   // dose = density at surface
    double d = (side == LEFT) ? i2x(i) : T - i2x(i);
    D[i] += dose * exp(-d/depth);
  }
}

void ConstantDope (TImplantSide side, double dose, double depth)
{
  for (int i=0; i<=N; i++) {                   // dose = dopant density in volume
    double d = (side == LEFT) ? i2x(i) : T - i2x(i);
    if (d<depth) D[i] += dose;
  }
}

void __fastcall TForm1::DefineDoping(TObject *Sender)
{
  // in this routine, all dopings are in atoms/cm3 for
  // compatibility with standard nomenclature

  float WaferDoping = LEWaferDoping->Text.ToDouble(); // <0 for p-wafer
  for (int i=0; i<=N; i++) D[i] = WaferDoping;

  T = LEThickness->Text.ToDouble();            // get silicon thickness from GUI

  if (RB_Dope_PN->Checked) {                   // symmetric pn junction
    ConstantDope (LEFT,  -1e15, T/2);
    ConstantDope (RIGHT, +1e15, T/2);
  } else if (RB_Dope_PN_assymetric->Checked) { // assymetric pn junction
    ConstantDope (LEFT,  -1e14, T/2);
    ConstantDope (RIGHT, +1e15, T/2);
  } else if (RB_Dope_PIN->Checked) {           // PiN diode
    Diffuse      (LEFT,  - 1.0e18, 0.002 * T);
    Diffuse      (RIGHT,   1.0e18, 0.002 * T);
  } else if (RB_Dope_SpecialPiN->Checked) {    // PiN diode with thick implants
    ConstantDope (LEFT,  - 1.0e18, 0.2 * T);
    ConstantDope (RIGHT,   1.0e18, 0.2 * T);
  } else {                                     // APD
    Diffuse      (LEFT,  - 1.0e18, 0.003 * T);
    Diffuse      (RIGHT, - 1.0e16, 0.02  * T); // 0.02
    Diffuse      (RIGHT,   1.0e18, 0.003 * T); // 0.01
   }

  for (int i=0; i<=N; i++) D[i] *= 1e-12;      // rescale from cm^(-3) to um^(-3)

  // smear the profile a bit (drive-in diffusion)
  for (int i=1; i<N; i++) Phi[i] = (D[i+1] + D[i-1])/2; // Use Phi as temp. array
  for (int i=1; i<N; i++) D[i] = Phi[i];

  ChartDoping->Series[0]->Clear();             // Plot the doping profile
  ChartDoping->Title->Text->Clear();
  ChartDoping->Title->Text->Add("log10(Doping [atoms/um3])");;
  for (int i=0; i<=N; i++) {
    float dop = D[i];
    float val = (dop > 0) ? log10(dop) : -log10(-dop);
    ChartDoping->Series[0]->AddXY(i2x(i), val, "", clBlack);
  }
}


//**********************************************************//
// Main stuff with GUI interaction starts here              //
//**********************************************************//

__fastcall TForm1::TForm1(TComponent* Owner): TForm(Owner)
{
  DefineDoping(this);                            // start with an initial doping
}

void __fastcall TForm1::ButtonStartClick(TObject *Sender)
{
  V  = LEBiasVoltage->Text.ToDouble();           // get bias voltage from GUI
  Vp = V/(2*C_Uth);                              // rescale bias voltage (see eq. (22) and (23))

  T  = LEThickness->Text.ToDouble();             // get silicon thickness from GUI
  a  = -(Treal) (N * N) / (T * T);               // constant alpha in .tex document

  DefineDoping(this);                            // Define Doping Profile

  // Vbi = C_Uth * log(- D[0]*D[N] / (C_ni*C_ni));  // calculate built in voltage

  for (int i=0; i<=N; i++) {                     // Guess initial potential
    //  Phi[i] = C_Uth * ArSinh(D[i] / (2 * C_ni) * exp(Vp));  // direct equation (23)
    if (D[i]>0)  // approximate the ArSinh so that we get a result for large V
      Phi[i] =   C_Uth * ( log( D[i] / C_ni) + Vp );
    else
      Phi[i] = - C_Uth * ( log( - D[i] / C_ni) + Vp );
  }
  // Note that the difference of Phi equals Vp:
  // Dopings are different on both sides. Assume D[N]>0, D[0]<0:
  // Delta Phi = Phi[N] - Phi[0]
  //           = C_Uth * (log( D[N] / C_ni) + Vp + log( |D[0]| / C_ni) + Vp)
  //           = C_Uth * 2 * Vp + C_Uth * log ( - D[N] D[0] / (C_ni * C_ni) )
  //           = <--   V    -->           <---    = Vbi / C_Uth          -->
  //           = V + Vbi

  // rescale Doping profile for faster computation, equation (9)
  for (int i=0; i<=N; i++) D[i] *= - C_q / C_eps;

  // Print the start values
  /*
  Memo1->Lines->Clear();
  Memo1->Lines->Add("Thermal  voltage = " + FloatToStrF(1000*C_Uth,ffFixed,5,3) + " mV");
  Memo1->Lines->Add("Build in voltage = " + FloatToStrF(1000*Vbi,  ffFixed,5,3) + " mV");
  */
  PlotPotential (ChartCurr);
}

void __fastcall TForm1::ButtonIterateClick(TObject *Sender)
{
  double err2 = Iterate(true, true);
  Memo1->Lines->Add("err2 = " + FloatToStrF(err2,ffExponent,5,3));
  ButtonDrift->Enabled = true;
}

double __fastcall TForm1::Iterate(bool do_print, bool do_plot)
{
  // At this point, Phi[i] and D[i] are defined

  Treal X[N+1],B[N+1];                           // intermediate terms
  Treal S[N+1],U[N+1],Y[N+1];                    // intermediate terms (see script)
  Treal Lambda[N+1];                             // error term

  for (int i=1; i<N; i++) {                      // not calculated for i=0, i=N
// original:
//  X[i] = C_C / C_Uth * exp(-Vp) * cosh(Phi[i]/C_Uth) - 2 * a;
//  B[i] = - a * (Phi[i-1]+Phi[i+1]-2*Phi[i]) - D[i] - C_C * exp(-Vp) * sinh(Phi[i]/C_Uth);

// change this a bit for numerical solution for large Vp
    X[i] = C_C / C_Uth / 2 * (exp(Phi[i]/C_Uth - Vp)+ exp(-Phi[i]/C_Uth-Vp)) - 2 * a;
    B[i] = - a * (Phi[i-1]+Phi[i+1]-2*Phi[i]) - D[i]
           - C_C / 2 * (exp(Phi[i]/C_Uth-Vp) - exp(-Phi[i]/C_Uth-Vp) );
  }
  X[N] = X[N-1];
  B[N] = B[N-1];


  // Eliminate elements below diagonal. Index = row to change
  // X[j] = M_jj from paper
  // note that a = - 1 / Delta^2
  S[1] = X[1];
  for (int j=2; j<=N; j++) S[j] = X[j] - a * a / S[j-1];

  // B[j] = R_j from paper
  U[1] = B[1];
  for (int j=2; j<=N; j++) U[j] = B[j] - a * U[j-1] / S[j-1] ;

  // Eliminate elements above diagonal. Index = row to change
  Y[N] = U[N];
  for (int j=N-1; j>=1; j--) Y[j] = U[j] - a * U[j+1] / S[j+1];

  // calculate Lambda
  Lambda[0] = 0;
  for (int j=1; j<=N; j++) Lambda[j] = Y[j] / S[j];

  // update Phi
  for (int j=0; j<=N; j++) Phi[j] += Lambda[j];

  // calculate E (for plotting only)
  for (int j=1; j< N; j++) E[j] = (Phi[j]-Phi[j-1])/T*N;
  E[0] = E[1]; E[N] = E[N-1];

  if (do_plot) {
    PlotPotential(ChartCurr);
    PlotField(ChartField);
    PlotCarrierDensity(ChartInduced);
  }

  Treal err2 = 0;
  for (int j=0; j<=N; j++) err2 += Lambda[j] * Lambda[j];
  return err2;
}

void __fastcall TForm1::ButtonSolveClick(TObject *Sender)
{
  const double err2_limit = 1e-22;
  ButtonStartClick(this);
  do {} while (Iterate(false, false) > err2_limit);
  Memo1->Lines->Add("err2 = " + FloatToStrF(Iterate(false, true),ffExponent,5,3));
  ButtonDrift->Enabled = true;
}

// Drift the particles and amplify charge
// The code is very brute force: It would better to delete particles
// that have arrived from the list.

void __fastcall TForm1::ButtonDriftClick(TObject *Sender)
{
  const int NTRACK=100;                          // only for track

  std::list <TParticle> PL;                      // particle list
  PL.clear();
  TParticle particle;                            // prototypes

  float x0 = LEStartPos->Text.ToDouble();        // get start position from GUI
  if (RBElectron->Checked || RBElectronHole->Checked) {
    particle.pos = x0; particle.type=ELECTRON;
    PL.push_back(particle);
  }
  if (RBHole->Checked || RBElectronHole->Checked) {
    particle.pos = x0; particle.type=HOLE;
    PL.push_back(particle);
  }
  if (RBTrack->Checked)
    for (int i=0; i<NTRACK; i++) {
      float x = T / (float) NTRACK * ((float) i + 0.5);
      particle.pos = x; particle.type=HOLE;
      PL.push_back(particle);
      particle.pos = x; particle.type=ELECTRON;
      PL.push_back(particle);
    }

  Memo1->Clear();
  ChartInduced->Series[0]->Clear();
  ChartInduced->Title->Text->Clear();
  ChartInduced->Title->Text->Add("induced current [nA] vs. time [ns]");;

  unsigned int NPart = 1;                        // count particles here
  float Qtotal = 0;
  for (float time = 0; time < tmax; time += dt) {
    float dQ = 0;                  // add up charge contributions from all particles in this time step

    for (std::list<TParticle>::iterator iter = PL.begin(); iter != PL.end(); ) {

      float Efield = E[x2i(iter->pos)];
      float v = vdrift(iter->type, Efield);
      float dx = v * dt;
      float posnew = iter->pos + dx;

      if (posnew <= 0) {         // particle hits left electrode
        dQ += iter->pos;         // only drift in sensors induces signal
        iter = PL.erase(iter);   // delete this from the list
      } else if (posnew >= T) {  // particle hits the right electrode
        dQ += (T - iter->pos);   // only this drift path counts
        iter = PL.erase(iter);   // delete it
      } else {
        dQ += fabs(dx);
        iter->pos = posnew;
        // create new particles here...
        if (iter->type==ELECTRON) {
          if (IonizationProbability(Efield) > (float) rand() / (float) RAND_MAX) {
            particle.pos = posnew;
            particle.type=ELECTRON;
            PL.push_front(particle);   // push to front (why?)
            particle.pos = posnew;
            particle.type= HOLE;
            PL.push_front(particle);
            NPart++;
            //Memo1->Lines->Add("Create pair at " + FloatToStrF(posnew,ffFixed,5,2));
          }
        }
        iter++;
      } // for
      if (NPart > 100000) break;
    }
    dQ /= T;                        // normalize to thickness of detector
    // plot current:  dQ*C_q is charge in Coulombs, (dt/1e9) is time in ns
    ChartInduced->Series[0]->AddXY(time, dQ*C_q / (dt/1e9) * 1e9,"",clBlue);
    Qtotal += dQ;
  } // for time
  Memo1->Lines->Add("Collected Charge: " + FloatToStrF(Qtotal,ffFixed,5,2));
  Memo1->Lines->Add("Gain:             " + AnsiString(NPart) );
  Memo1->Lines->Add("Not arrived:      " + AnsiString(PL.size()) );

  std::list<TParticle>::iterator iter = PL.begin();
  for ( int i=0; i<10; i++ ) {
    Memo1->Lines->Add("pos:" + AnsiString(iter->pos) );
    iter++;
  }


}

void __fastcall TForm1::RBElectronClick(TObject *Sender)
{
  LEStartPos->Enabled=true;
}

void __fastcall TForm1::RBHoleClick(TObject *Sender)
{
  LEStartPos->Enabled=true;
}

void __fastcall TForm1::RBElectronHoleClick(TObject *Sender)
{
  LEStartPos->Enabled=true;
}

void __fastcall TForm1::RBTrackClick(TObject *Sender)
{
  LEStartPos->Enabled=false;
}
